@@ -7,7 +7,9 @@ use warnings;
 
 =head1 NAME
 
-Net::ACME2 - IETF-standard ACME client protocol in Perl
+Net::ACME2 - L<IETF|https://www.ietf.org>-standard ACME (Let’s Encrypt) client
+
+X<Lets Encrypt> X<Let's Encrypt> X<letsencrypt>
 
 =head1 SYNOPSIS
 
@@ -52,7 +54,7 @@ Net::ACME2 - IETF-standard ACME client protocol in Perl
 
     $acme->accept_challenge($challenge);
 
-    sleep 1 while !$acme->poll_authorization($authz);
+    sleep 1 while 'valid' ne $acme->poll_authorization($authz);
 
     # ... Make a key and CSR for *.example.com
 
@@ -74,20 +76,41 @@ L<Net::ACME2::LetsEncrypt>.
 
 =head1 DESCRIPTION
 
-This module implements client logic for the IETF-standard ACME
+This library implements client logic for the L<IETF|https://www.ietf.org>-standard ACME
 (Automated Certificate Management Environment) protocol. As of this
 writing, that protocol remains in development; the latest draft will
 be available from L<https://datatracker.ietf.org/doc/draft-ietf-acme-acme/>.
 
-This module derives from L<Net::ACME>, which implements the
+Net::ACME2 derives from L<Net::ACME>, which implements the
 (significantly different) earlier draft of that protocol as initially
 deployed by L<Let’s Encrypt|http://letsencrypt.org>.
 
+Net::ACME2 is pure Perl, and all of its dependencies are either pure Perl
+or core modules.
+
 =head1 STATUS
 
-This is a “first-stab”, alpha-grade implementation. This
-distribution, like the underlying protocol, is very much still
-subject to change.
+This is a beta-grade implementation. While the underlying protocol
+is L<in use for production|https://community.letsencrypt.org/t/acme-v2-production-environment-wildcards/55578>,
+it’s still not finalized; consequently, this distribution remains
+subject to change. It is expected that any further breaking changes
+will be small, but you still B<MUST> check the changelog before upgrading!
+
+=head1 FEATURES
+
+=over
+
+=item * This is a pure-Perl solution, all of whose dependencies are either
+core modules or pure-Perl themselves. Net::ACME2 will run anywhere that Perl
+runs. :)
+
+=item * Support for both ECDSA and RSA encrytion.
+
+=item * Support for http-01, dns-01, and tls-alpn-01 challenges.
+
+=item * Comprehensive error handling with typed, L<X::Tiny>-based exceptions.
+
+=back
 
 =head1 ERROR HANDLING
 
@@ -97,16 +120,14 @@ Specific error classes aren’t yet defined.
 =cut
 
 use Crypt::Format ();
+use Crypt::Perl::PK ();
 use MIME::Base64 ();
-use Types::Serialiser ();
 
-use Net::ACME2::Constants ();
 use Net::ACME2::HTTP ();
 use Net::ACME2::Order ();
 use Net::ACME2::Authorization ();
 
-our $VERSION;
-*VERSION = *Net::ACME2::Constants::VERSION;
+our $VERSION = '0.2';
 
 use constant {
     JWS_FORMAT => undef,
@@ -114,11 +135,14 @@ use constant {
     _JWK_THUMBPRINT_DIGEST => 'sha256',
 };
 
+# accessed from test
 use constant newAccount_booleans => qw(
     termsOfServiceAgreed
     onlyReturnExisting
 );
 
+# the list of methods that need a “jwk” in their JWS Protected Header
+# (cf. section 6.2 of the spec)
 use constant FULL_JWT_METHODS => qw(
     newAccount
     revokeCert
@@ -134,13 +158,13 @@ interactions with the ACME server. %OPTS is:
 =over
 
 =item * C<key> - Required. The private key to associate with the ACME2
-user. PEM or DER format.
+user. Anything that C<Crypt::Perl::PK::parse_key()> can parse is acceptable.
 
 =item * C<key_id> - Optional. As returned by C<key_id()>.
 Saves a round-trip to the ACME2 server, so you should give this
 if you have it.
 
-=item * <directory> - Optional. A hash reference to use as the
+=item * C<directory> - Optional. A hash reference to use as the
 directory contents. Saves a round-trip to the ACME2 server, but there’s
 no built-in logic to determine when the cache goes invalid. Caveat
 emptor.
@@ -152,7 +176,7 @@ emptor.
 sub new {
     my ( $class, %opts ) = @_;
 
-    die 'Need “key”!' if !$opts{'key'};
+    _die_generic('Need “key”!') if !$opts{'key'};
 
     my $self = {
         _host => $class->HOST(),
@@ -202,8 +226,11 @@ sub get_terms_of_service {
     }
 
     my $dir = $self->_get_directory();
-    my $url = $self->_get_directory()->{'meta'} or die 'No “meta” in directory!';
-    $url = $url->{'termsOfService'} or die 'No “terms-of-service” in directory metadata!';
+
+    # Exceptions here indicate an ACME violation and should be
+    # practically nonexistent.
+    my $url = $dir->{'meta'} or _die_generic('No “meta” in directory!');
+    $url = $url->{'termsOfService'} or _die_generic('No “termsOfService” in directory metadata!');
 
     return $url;
 }
@@ -222,7 +249,10 @@ or 0 if the account already existed.
 sub create_new_account {
     my ($self, %opts) = @_;
 
-    $opts{$_} &&= Types::Serialiser::true() for newAccount_booleans();
+    for my $name (newAccount_booleans()) {
+        next if !exists $opts{$name};
+        ($opts{$name} &&= JSON::true()) ||= JSON::false();
+    }
 
     my $resp = $self->_post(
         'newAccount',
@@ -240,7 +270,10 @@ sub create_new_account {
     my $struct = $resp->content_struct();
 
     if ($struct) {
-        $struct->{$_} = !!$struct->{$_} for newAccount_booleans();
+        for my $name (newAccount_booleans()) {
+            next if !exists $struct->{$name};
+            ($struct->{$name} &&= 1) ||= 0;
+        }
     }
 
     return 1;
@@ -287,7 +320,7 @@ sub create_new_order {
 Fetches the authorization’s information based on the given URL
 and returns a L<Net::ACME2::Authorization> object.
 
-The URL is as given by L<Net::ACME2::Order>.
+The URL is as given by L<Net::ACME2::Order>’s C<authorizations()> method.
 
 =cut
 
@@ -333,7 +366,8 @@ sub get_authorization {
 
 =head2 I<OBJ>->make_key_authorization( CHALLENGE )
 
-Accepts an instance of L<Net::ACME2::Challenge> and returns
+Accepts an instance of L<Net::ACME2::Challenge> (probably a subclass
+thereof) and returns
 a key authorization string suitable for handling the given CHALLENGE.
 See F</examples> in the distribution for example usage.
 
@@ -347,7 +381,7 @@ handle HTTP challenges.
 sub make_key_authorization {
     my ($self, $challenge_obj) = @_;
 
-    die 'Need a challenge object!' if !$challenge_obj;
+    _die_generic('Need a challenge object!') if !$challenge_obj;
 
     return $challenge_obj->token() . '.' . $self->_key_thumbprint();
 }
@@ -361,7 +395,7 @@ Signal to the ACME server that the CHALLENGE is ready.
 sub accept_challenge {
     my ($self, $challenge_obj) = @_;
 
-    my $post = $self->_post_url(
+    $self->_post_url(
         $challenge_obj->url(),
         {
             keyAuthorization => $self->make_key_authorization($challenge_obj),
@@ -371,24 +405,27 @@ sub accept_challenge {
     return;
 }
 
-=head2 I<OBJ>->poll_authorization( AUTHORIZATION )
+=head2 $status = I<OBJ>->poll_authorization( AUTHORIZATION )
 
 Accepts a L<Net::ACME2::Authorization> instance and polls the
 ACME server for that authorization’s status. The AUTHORIZATION
 object is then updated with the results of the poll.
 
+As a courtesy, this returns the object’s new C<status()>.
+
 =cut
 
 #This has to handle updates to the authz and challenge objects
 *poll_authorization = *_poll_order_or_authz;
 
-=head2 I<OBJ>->finalize_order( ORDER, CSR )
+=head2 $status = I<OBJ>->finalize_order( ORDER, CSR )
 
 Finalizes an order and updates the ORDER object with the returned
 status. The CSR may be in either DER or PEM format.
 
-ORDER may have C<status()> of C<valid> after this operation,
-or you may need to C<poll_order()>.
+As a courtesy, this returns the ORDER’s C<status()>. If this does
+not equal C<valid>, then you should probably C<poll_order()>
+until it does.
 
 =cut
 
@@ -416,7 +453,7 @@ sub finalize_order {
 
     $order_obj->update($content);
 
-    return;
+    return $order_obj->status();
 }
 
 =head2 I<OBJ>->poll_order( ORDER )
@@ -439,17 +476,25 @@ sub _key_thumbprint {
 sub _get_directory {
     my ($self) = @_;
 
-    return $self->{'_directory'} ||= do {
+    $self->{'_directory'} ||= do {
         my $dir_path = $self->DIRECTORY_PATH();
         $self->{'_ua'}->get("https://$self->{'_host'}$dir_path")->content_struct();
     };
+
+    my $new_nonce_url = $self->{'_directory'}{'newNonce'} or do {
+        _die_generic('Directory is missing “newNonce”.');
+    };
+
+    $self->{'_ua'}->set_new_nonce_url( $new_nonce_url );
+
+    return $self->{'_directory'};
 }
 
 sub _require_key_id {
     my ($self, $opts_hr) = @_;
 
     $opts_hr->{'_key_id'} = $self->{'_key_id'} or do {
-        die 'No key ID has been set. Either pass “key_id” to new(), or create_new_account().';
+        _die_generic('No key ID has been set. Either pass “key_id” to new(), or create_new_account().');
     };
 
     return
@@ -464,13 +509,7 @@ sub _poll_order_or_authz {
 
     $order_or_authz_obj->update($content);
 
-    my $status = $order_or_authz_obj->status();
-
-    return 1 if $status eq 'valid';
-    return 0 if $status eq 'pending' || $status eq 'processing';
-
-    use Data::Dumper;
-    die Dumper $order_or_authz_obj; #TODO
+    return $order_or_authz_obj->status();
 }
 
 sub _key_obj {
@@ -499,7 +538,9 @@ sub _post {
     my $post_method;
     $post_method = 'post_full_jwt' if grep { $link_name eq $_ } FULL_JWT_METHODS();
 
-    my $url = $self->_get_directory()->{$link_name} or die "Unknown link name: “$link_name”";
+    # Since the $link_name will come from elsewhere in this module
+    # there really shouldn’t be an error here, but just in case.
+    my $url = $self->_get_directory()->{$link_name} or _die_generic("Unknown link name: “$link_name”");
 
     return $self->_post_url( $url, $data, $post_method );
 }
@@ -516,18 +557,29 @@ sub _post_url {
     return $self->{'_ua'}->$post_method( $url, $data );
 }
 
+sub _die_generic {
+    die Net::ACME2::X->create('Generic', @_);
+}
+
 1;
 
 =head1 TODO
 
 =over
 
-=item * Re-evaluate once there is a production server in place.
+=item * Add pre-authorization support if there is ever a production
+use for it.
 
-=item * Tighten up challenge failure response.
+=item * Expose the Retry-After header via the module API.
 
-=item * Add tests.
+=item * Add (more) tests.
 
 =back
 
+=head1 SEE ALSO
+
+L<Crypt::Perl> provides pure-Perl cryptography for this library. See the
+present library distribution’s F</examples> directory for sample usage
+to generate keys and CSRs.
+
 =cut
